#ifndef __NETWORK_H
#define __NETWORK_H

#include "definitions.h"

#ifdef __NETWORK_SUPPORT__

#include "SDL_net.h"

#include "console.h"
#ifdef __NCURSES_SUPPORT__
#include "nconsole.h"
#endif

class Network
{
	public:
	struct host
	{
		char   alias   [ 256 ] ;
		char   key     [ 256 ] ;
		char   address [ 256 ] ;
		Uint32 timeout         ; // timestamp of death :)
		Uint32 speed           ; // respond time in ms
	} ;

	struct simple_host
	{
		char address [ 256 ] ;
		Uint16 port ;
	} ;
	
	struct peer
	{
		char      address [256] ; // peer ip
		TCPsocket socket        ; // our socket :)
		IPaddress ip            ; // ip and port
		bool      established   ; // connection established?
		Uint32    timeout       ; // time till starved
		Uint32    born          ; // since when?
		GlDynArray < host > known_hosts ; // seen hosts
		char outgoing_name [256]; // known as this to others
		
		char   key      [ 256 ] ; // a unique id from the sender (!)
		char   alias    [ 256 ] ; // aliasname for the host
	} ;
	
	struct packet // 8k max size !
	{
		uint   type     ; // type of packet
		char   from     [ 256 ] ; // sender ip
		char   dest     [ 256 ] ; // destination ip
		char   from_key [ 256 ] ; // a unique id from the sender (!)
		char   dest_key [ 256 ] ; // a unique id from the recepient (!)
		char   alias    [ 256 ] ; // aliasname for the host
		char   hash     [ 256 ] ; // a unique id for this packet
		uint   hops     ; // how far the packet as travelled
		uint   max_hops ; // we might want a distance limit...
		char   data     [ 4096 ] ; // should NOT be bigger than 4k !
	} ;

	enum
	{
		PACKET_TYPE_HANDSHAKE,
		PACKET_TYPE_PING,
		PACKET_TYPE_ALIVE,
		PACKET_TYPE_USERDATA,
		PACKET_TYPE_GAMEDATA,
		PACKET_TYPE_MESSAGE,
		PACKET_TYPE_PMESSAGE,
		PACKET_TYPE_DISCONNECTED
	} ;
	
	void Init () ;
	
	void EnableDebugging () ;
	void DisableDebugging () ;

	void AddHostToConnectionList ( simple_host *host ) ;
	void ConnectToServersInList () ;
	
	void EnableConsolePrinting ( Console *new_console, Console *new_buddy_list ) ;
	void DisableConsolePrinting () ;


	const char *Cached_SDLNet_ResolveIP ( IPaddress *ip ) ;
	int Cached_SDLNet_ResolveHost ( IPaddress *address,
					const char *host,
					Uint16 port ) ;
	
	void GetIPAsString ( IPaddress *ip, char *return_string ) ;
	void ResolveIP ( IPaddress *ip, char *return_hostname ) ;
	char *ResolveIP ( char *address ) ;
	
	// debug printing
	void printd ( const char *format, ... ) ;
	// console printing
	void Printf ( const char *format, ... ) ;
	
	std::string AddUsernameAtBeginning ( std::string text ) ;
	
	bool AddressIsKnownToPeer ( peer *peer, char *address ) ;
	bool KeyIsKnownToPeer ( peer *peer, char *key ) ;
	
	int SendPacket ( peer *peer, packet *packet ) ;
	int SendMessage ( char *address, char *message,
						uint packet_type = PACKET_TYPE_MESSAGE ) ;
	
	int  FindPeerBySocketI ( TCPsocket socket ) ;
	peer *FindPeerBySocket ( TCPsocket socket ) ;
	
	void ConnectTo ( char *client, int port = 5555 ) ;
	void ListenOnPort ( int port = 5555 ) ;
	void Disconnect ( peer *peer ) ;

	/* obsolete */
	void StartAsServer ( int port ) ;
	void StartAsClient ( char *server, int port ) ;
	int  Connect () ;
	void Disconnect () ;
	/*          */
	
	void SendHandshake ( TCPsocket socket ) ;
	void SendPing ( char *address ) ;
	
	void TCPAcceptEvent ( TCPsocket socket ) ;
	bool TCPReceiveEvent ( peer *peer ) ; // returns true = alive ; false = died
	void TCPCloseEvent ( peer *peer ) ;
	
	
	// buddy list thingies
	host *FindBuddyByKey ( char *buddy_key ) ;
	host *AddBuddy ( char *alias, char *key, char *from_address ) ;
	bool DelBuddy ( char *key ) ;
	host *RefreshBuddy ( char *key ) ;
	host *ChangeBuddyName ( char *key, char *new_name ) ;
	void UpdateBuddyList () ;
	//
	
	
	void ProcessEvents () ;
	
	char *StripHeadAndTail ( char *text ) ;
	char *StripMarks ( char *text ) ;
	GlDynArray < std::string > *GetArgList ( char *string,
												char delimiter = ' ' ) ;
	// returns false if it's a global message -> for all peers
	bool ProcessMessage ( peer *peer, char *t_message ) ;
	
	void UpdateNConsole () ;
	
	void Quit () ;
	
	private:
	GlDynArray < peer > *peer_list   ;
	
	struct known_packet
	{
		char hash [ 256 ] ;
		Uint32 timeout ;
	} ;
	GlDynArray < known_packet > *known_packets ;
	
	peer self ;
	
	Console *console ;
	Console *buddy_list ;
#ifdef __NCURSES_SUPPORT__
	NConsole *nconsole ;
#endif
	
	GlDynArray < host > *buddies ;
	
	SDLNet_SocketSet socketset ;

	GlDynArray < simple_host > connectto_list ;

	GlDynArray < peer > dnscache ;
} ;

#endif

#endif
